# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2007 Fluendo S.A.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

EULA = """
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this program; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
"""

import gobject
import gtk

from codeina.extern.log import log

from gettext import gettext as _

# a standard MessageDialog is not resizable and too small for most EULA's
class EulaDialog(gtk.Window, log.Loggable):
    def __init__(self, parent, text):
        gtk.Window.__init__(self)

        # set up our window
        self.set_modal(True)
        self.set_transient_for(parent)
        self.set_border_width(12)
        self.set_icon(parent.get_icon())
        self.set_default_size(400, 600)

        self.response = gtk.RESPONSE_CANCEL

        # our internal hbox
        self._hbox = gtk.HBox()
        self.add(self._hbox)

        # top part is the vbox, with title and text
        self.vbox = gtk.VBox()
        self.vbox.set_spacing(12)
        self._hbox.add(self.vbox)

        self.vbox.pack_start(gtk.Label(
            _("Please read the following "
                "end-user license agreement.")), False, True, 6)

        # let's reflow the text
        blocks = text.strip().split('\n\n')
        scrubbed = []
        for block in blocks:
            scrubbed.append(" ".join(block.split('\n')))
        text = "\n\n".join(scrubbed)

        b = gtk.TextBuffer()
        b.set_text(text)
        view = gtk.TextView(b)
        view.set_wrap_mode(gtk.WRAP_WORD)
        view.set_editable(False)
        view.set_cursor_visible(False)

        w = gtk.ScrolledWindow()
        w.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        w.add(view)
        self.vbox.add(w)

        # bottom part is the action area
        self.action_area = gtk.HBox()
        self.vbox.pack_end(self.action_area, False, True, 0)

        self._refuse = gtk.Button(_("_Refuse"))
        i = gtk.Image()
        i.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
        self._refuse.set_image(i)
        self._refuse.connect('clicked', self.clicked)
        # FIXME: how to default to refuse ?
        self.set_focus(self._refuse)
        self.action_area.pack_start(self._refuse, False, True, 0)

        self._accept = gtk.Button(_("_Accept"))
        i = gtk.Image()
        i.set_from_stock(gtk.STOCK_APPLY, gtk.ICON_SIZE_BUTTON)
        self._accept.set_image(i)
        self._accept.connect('clicked', self.clicked)
        self.action_area.pack_end(self._accept, False, True, 0)

        self.result = gtk.RESPONSE_CANCEL

        # As per HIG, empty dialogue title
        self.set_title(_(''))
        self.show_all()

        self._main = gobject.MainLoop()

        self.connect('destroy', lambda w: self._main.quit)

    def clicked(self, button):
        if button == self._accept:
            self.response = gtk.RESPONSE_OK
        self._main.quit()

    def run(self):
        self._main.run()
        self.debug('Dialog done, returning %r', self.response)
        return self.response

if __name__ == '__main__':
    w = gtk.Window()
    dialog = EulaDialog(w, EULA)
    ret = dialog.run()
    if ret == gtk.RESPONSE_OK:
        print "Accepted"
    else:
        print "Refused"
    
