# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2007 Fluendo S.A.
#
# Written by Tim-Philipp Müller <tim centricular net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Inspect a .tar.gz or .tar.bz2 file containing GStreamer plugins and print
# the appropriate "provides" line for the codeina available-plugins.xml file

import os
import tarfile
import tempfile
import time
import sys


def _inspect_plugin_file(plugin_file_path):
    child = os.popen("gst-inspect-0.10" +
                     " --print-plugin-auto-install-info" + " '%s'" %
                     plugin_file_path)
    output = child.read()
    if not child.close():
        provides = ''
        sep = ''
        for line in output.splitlines():
            line = line.strip()
            if len(line) > 0:
                provides = provides + sep + line
                sep = '; '
        return provides.strip()
    else:
        return ''


def _inspect_plugin_archive(archive_path):
    try:
        tar = tarfile.open(archive_path, 'r')
    except:
        print "Error opening tar archive", archive_path
        return None

    basename = os.path.basename(archive_path)
    if basename.endswith('.tar.gz'):
        basename = basename[0:-7]
    elif basename.endswith('.tar.bz2'):
        basename = basename[0:-8]

# create temporary directory

    try:
        tdir = tempfile.mkdtemp('', basename + '.dir.')
    except:
        print "Failed to create temporary directory"
        return None

# extract and inspect each plugin file in the tar archive

    max_mtime = 0
    all_provides = {}

    for name in tar.getnames():
        if name.endswith('.so') \
        or name.endswith('.sl') \
        or name.endswith ('.dll') \
        or name.endswith ('.DLL') \
        or name.endswith ('.dynlib'):
            try:
                info = tar.getmember(name)
                if info.mtime > max_mtime:
                    max_mtime = info.mtime
                tar.extract(name, tdir)
                provides = _inspect_plugin_file("%s/%s" % (tdir, name))
                if provides:
                    all_provides[name] = provides
            except:
                print "Failed to extract or inspect", name

# close tar file and remove temporary directory

    tar.close()
    os.system("rm -rf '%s'" % tdir)

# concatenate result into one single provides statement

    provides_line = ''
    sep = ''
    for value in all_provides.values():
        if value:
            provides_line = provides_line + sep + value
            sep = '; '

    max_mtime_str = time.strftime('%Y-%m-%d', time.gmtime(max_mtime))

    return provides_line, max_mtime_str


# if program is called directly, inspect the given archive file(s)

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print "Usage: %s [PLUGINS_BUNDLE.TAR.GZ|PLUGINS_BUNDLE.TAR.BZ2]" % \
              (sys.argv)[0]
        sys.exit(1)

    for archive in (sys.argv)[1:]:
        basename = os.path.basename(archive)

        print "*****", basename
        provides_line, max_mtime_str = _inspect_plugin_archive(archive)
        print "provides='%s'" % provides_line
        print "last_modified='%s'" % max_mtime_str
        print ''

    sys.exit(0)

