# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
#
# Written by Tim-Philipp Müller <tim centricular net> and others
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import os
import sys
import traceback
import check
from xdg import BaseDirectory
from xdg import DesktopEntry

#############################################################################
# env_var_is_set
#############################################################################


def env_var_is_set(env_var_name):
    try:
        if (os.environ)[env_var_name] and (os.environ)[env_var_name] != \
            '':
            return True
        else:
            return False
    except KeyError:
        return False


#############################################################################
# open_url_in_browser (FIXME: implement properly)
#############################################################################


def open_url_in_browser(link):
    command = None
    if env_var_is_set('GNOME_DESKTOP_SESSION_ID'):
        command = 'gnome-open'
    elif env_var_is_set('KDE_FULL_SESSION'):
        command = 'kfmexec'
    elif env_var_is_set('BROWSER'):

    # tough luck if someone sets this to a console browser ...

        command = (os.environ)['BROWSER']
    if command:
        try:
            os.spawnvpe(os.P_NOWAIT, command, [command, link], os.environ)
            return
        except:
            pass

  # fallback (x-www-browser is a debian/ubuntu thing)

    for browser in ['x-www-browser', 'firefox', 'mozilla-firefox',
                    'mozilla', 'mozilla-bin', 'epiphany', 'opera']:
        try:
            os.spawnvpe(os.P_NOWAIT, browser, [browser, link], os.environ)
            return
        except:
            pass
    print "Can't find working browser to open URI", link


#############################################################################
# scrub_filename
#############################################################################

def scrub_filename(filename):
    """
    Scrub the filename of everything before 'codeina' and' twisted' to make them shorter.
    """
    i = filename.rfind('codeina')
    if i != -1:
        filename = filename[i:]
    else:
        # only pure twisted, not flumotion.twisted
        i = filename.rfind('twisted')
        if i != -1:
            filename = filename[i:]
    
    return filename

#############################################################################
# get_exception_message
#############################################################################

def get_exception_message(exception, frame=-1, filename=None):
    """
    Return a short message based on an exception, useful for debugging.
    Tries to find where the exception was triggered.
    """
    stack = traceback.extract_tb(sys.exc_info()[2])
    if filename:
        stack = [f for f in stack if f[0].find(filename) > -1]
    #import code; code.interact(local=locals())
    (filename, line, func, text) = stack[frame]
    filename = scrub_filename(filename)
    exc = exception.__class__.__name__
    msg = ""
    # a shortcut to extract a useful message out of most flumotion errors
    # for now
    if str(exception):
        msg = ": %s" % str(exception)
    return "exception %(exc)s at %(filename)s:%(line)s: %(func)s()%(msg)s" % locals()

#############################################################################
# get_failure_message
#############################################################################

def get_failure_message(failure):
    """
    Return a short message based on L{twisted.python.failure.Failure}.
    Tries to find where the exception was triggered.
    """
    exc = str(failure.type)
    msg = failure.getErrorMessage()
    if len(failure.frames) == 0:
        return "failure %(exc)s: %(msg)s" % locals()

    (func, filename, line, some, other) = failure.frames[-1]
    filename = scrub_filename(filename)
    return "failure %(exc)s at %(filename)s:%(line)s: %(func)s(): %(msg)s" % locals()

#############################################################################
# append_system_details_to_url
#############################################################################

def append_system_details_to_url(url):
    """
    Adds ?distro=xyz&os=xyz&arch=xyz to the given url
    Doesn't do any quoting, but shouldn't need to either
    """
    return url + "?os=%s&distro=%s&arch=%s" % (check.OS_NAME,    \
                                               check.DISTRO_NAME,\
                                               check.ARCH_NAME)

#############################################################################
# _get_desktop_entries_dirs
#
# returns directories to check for .desktop files
#############################################################################

def _get_desktop_entries_dirs():
    dirs = []
    for d in BaseDirectory.xdg_data_dirs:
        dirs.append(os.path.join(d,'applications'))
    return dirs

#############################################################################
# get_application_name_from_execname
#
# Tries to get a nice localised name from the name of an executable
# (e.g. "totem" => "Movie Player"), by looking for a matching .desktop file
#############################################################################

def get_application_name_from_execname(execname):
  # First, look for an $(execname).desktop file
    for d in _get_desktop_entries_dirs():
        entry = DesktopEntry.DesktopEntry()
        try:
            desktop_fn = os.path.join(d, execname + '.desktop')
            entry.parse(desktop_fn)
            return entry.getName()
        except DesktopEntry.ParsingError, e:
            pass

  # If that didn't work, look at all .desktop files and check the 'exec' lines
    for d in _get_desktop_entries_dirs():
        try:
            names = os.listdir(d)
            for n in names:
                if n.endswith('.desktop'):
                    try:
                        entry = DesktopEntry.DesktopEntry()
                        desktop_fn = os.path.join(d, n)
                        entry.parse(desktop_fn)
                        e = entry.getExec()
                        idx = e.find(' ')
                        if idx > 0:
                            e = e[0:idx]
                        e = os.path.basename(e)
                        if e == execname:
                            return entry.getName()
                    except DesktopEntry.ParsingError, e:
                        pass
        except:
            pass

  # No luck: just return the name of the executable capitalised
    return execname.capitalize()

