# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2007 Fluendo S.A.
#
# Written by Tim-Philipp Müller <tim centricular net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

"""
Representation of a plugin requirement
(just so we don't have to deal with arrays of string arrays)
"""

import gst

class RequiredPlugin:

    valid_req_types = ('decoder', 'encoder', 'element', 'urisource', 'urisink')

    def __init__(self, desc_string=None):
        self.valid = False

        if desc_string and desc_string.startswith('gstreamer|0.10|'):
            details = desc_string.split('|')
            if len(details) >= 5:
                self.majorminor = details[1]
                self.application = details[2]
                self.description = details[3]
                self.req_string = details[4]
                idx = self.req_string.find('-')
                if idx > 0:
                    self.req_type = self.req_string[0:idx]
                    self.req_detail = self.req_string[idx+1:]

                    if self.req_type in self.valid_req_types:
                        self.valid = True

    def __repr__(self):
        return "<%s for %s> " % (self.__class__.__name__, self.req_string)


  # currently we're only interested in specialising mp3, we should probably
  # do something more generic later that covers other common formats too
    def is_mp3_decoder(self):
        if not self.valid or self.req_type != 'decoder':
            return False
        s = gst.structure_from_string(self.req_detail)
        if not s.has_name('audio/mpeg') or not s.has_field('mpegversion'):
            return False
        if s['mpegversion'] == 1:
            return True
        return False


if __name__ == "__main__":
    req1 = RequiredPlugin()
    req2 = RequiredPlugin('gstreamer|0.10|totem|MP3 Decoder|decoder-'\
                          'audio/mpeg, mpegversion=(int)1, layer=(int)3')
    req3 = RequiredPlugin('gstreamer|0.10|totem|Microsoft Media Server (MMS)'\
                          ' protocol source|urisource-mms')

    print "req1 - is_mp3=", req1.is_mp3_decoder()
    print "req2 - is_mp3=", req2.is_mp3_decoder()
    print "req3 - is_mp3=", req3.is_mp3_decoder()
